@@ -8,6 +8,10 @@ import numpy as np
 from pathlib import Path
 import pickle
 import logging
+from matplotlib import scale
+import itertools
+
+plt.set_cmap('jet')
 def getLogging():
     logging.basicConfig(
         format='%(asctime)s %(levelname)-8s %(message)s',
@@ -53,12 +57,13 @@ def merge_single_experiment_results(root_path,suffix="jjson"):
     # print(paths,root_path)
     df_result_cur=pd.DataFrame()
     for path in paths:
-#         print(path)
+        print(path)
         with open(path, "r") as read_file:
         #     print("Converting JSON encoded data into Python dictionary")
             developer = json.load(read_file)
+            # print(path,"path")
             pd_frame=pd.DataFrame(developer).fillna(np.nan)    
-        df_result_cur=df_result_cur.append(pd_frame)
+        df_result_cur=pd.concat([df_result_cur,pd_frame])
 #     print(df_result_cur)
     return df_result_cur
 
@@ -134,7 +139,7 @@ def latex_two_f(x, y):                      # this is a demo function that takes
         y="{:#.4G}".format(y)
     else:
         y="{:.3f}".format(y)
-    return "&"+str(x) + "\$_{("+str(y)+")}\$"        # concatenate them as str
+    return "&"+str(x) + "$_{("+str(y)+")}$"        # concatenate them as str
 vec_latex_two_f = np.vectorize(latex_two_f) 
 def latex_single_f_latex(x):                      # this is a demo function that takes in two ints and 
     if x>1:
@@ -217,7 +222,46 @@ plots_x_partition:str="iterations",groupby="iterations",ax=None,graph_param={})-
         gca=plot.gca()
         gca.set(**graph_param)
         plot.legend()
+
+
+def plot(name_results_pair:dict,errbar=True, ax=None,graph_param={},\
+                    desiredGradFairColorDict={})->None:
     
+    '''    
+        name_results_pair:{method_name:result_dataframe}
+    '''
+    
+    prop_cycle = plt.rcParams['axes.prop_cycle']
+    colors_list = prop_cycle.by_key()['color']
+    colors=itertools.cycle(colors_list)
+    marker = itertools.cycle((',', '+', '.', 'o', '*')) 
+    if ax:
+        plot=ax
+    else:
+        plot=plt
+    for algo_name in name_results_pair:
+            x,y=name_results_pair[algo_name]
+            y=np.array(y)
+            if len(y.shape)==1:
+                y=y[:,None]
+            yMean=y.mean(axis=1)
+            yStd=y.std(axis=1)
+            ndata=len(x)
+#             assert plots_y_partition in algo_result, algo_name+" doesn't contain the partition "+plots_y_partition
+
+            if desiredGradFairColorDict is None:
+                color=next(colors)
+            else:
+                color=desiredGradFairColorDict[algo_name]
+            if not errbar:
+                plot.plot(x,y, marker = next(marker),color=color, label=algo_name, markevery=ndata//3)
+            else:
+                plot.errorbar(x,yMean, yStd, marker = next(marker),color=color, label=algo_name, markevery=ndata//3)
+    if ax is None:
+        gca=plot.gca()
+        gca.set(**graph_param)
+        plot.legend()
+
 def paramIterationPlot(result:dict,metrics_name,step,ax=None,xlim=None,savepath=None):
     """
     This function plot the performance along different parameters.
@@ -244,7 +288,8 @@ def paramIterationPlot(result:dict,metrics_name,step,ax=None,xlim=None,savepath=
             plot.errorbar(param[ind],mean[ind],yerr=std[ind],marker = next(marker),label=key, markevery=ndata//3)
         else:
             plot.plot(param,result_metric,marker = next(marker),label=key)
-def TradeoffPlot(result:dict,metrics_pair,step,ax=None,xlim=None,savepath=None):
+def TradeoffPlot(result:dict,metrics_pair,step,\
+                    desiredGradFairColorDict={},ax=None,xlim=None,savepath=None):
     """
     This function plot the tradeoff performance.
     """
@@ -259,6 +304,56 @@ def TradeoffPlot(result:dict,metrics_pair,step,ax=None,xlim=None,savepath=None):
         yResult=np.array([np.mean(i)for i in result_list[2]])
         ndata=len(xResult)
         plot.plot(xResult,yResult,marker = next(marker),label=key, markevery=ndata//3)
+
+def RequirementPlot(result:dict,metrics_pair,step,\
+                    desiredGradFairColorDict={},ax=None,xlim=None,savepath=None):
+    """
+    This function plot the tradeoff performance.
+    """
+    if ax:
+        plot=ax
+    else:
+        plot=plt
+    prop_cycle = plt.rcParams['axes.prop_cycle']
+    colors_list = prop_cycle.by_key()['color']
+    colors=itertools.cycle(colors_list)
+    marker = itertools.cycle(('v', '^', "<",">","p",'x',"X","D", 'o', '*')) 
+    for key, value in result.items():
+        result_list=extractResultWithParam(value,metrics_pair,step)
+        xResult=np.array([np.mean(i)for i in result_list[1]])
+        yResult=np.array([np.mean(i)for i in result_list[2]])
+        ndata=len(xResult)
+        xSortedid=np.argsort(xResult)
+        xResult=xResult[xSortedid]
+        yResult=yResult[xSortedid]
+        yResult=np.array([np.max(yResult[:i+1])for i in range(ndata)])
+        color=next(colors)
+        if key in desiredGradFairColorDict:
+            color=desiredGradFairColorDict[key]
+        plot.plot(xResult,yResult,color=color,marker = next(marker),label=key, markevery=ndata//3)
+
+        
+        
+def TradeoffScatter(result:dict,metrics_pair,step,\
+                    desiredGradFairColorDict={},ax=None,xlim=None,savepath=None):
+    """
+    This function plot the tradeoff performance.
+    """
+    if ax:
+        plot=ax
+    else:
+        plot=plt
+    marker = itertools.cycle(("o")) 
+    for key, value in result.items():
+        result_list=extractResult(value,metrics_pair,step)
+        xResult=np.mean(result_list[0])
+        yResult=np.mean(result_list[1])
+        # ndata=len(xResult)
+        if key in desiredGradFairColorDict:
+            color=desiredGradFairColorDict[key]
+            plot.scatter(xResult,yResult,color=color,marker = next(marker),label=key,s=200)
+        else:
+            plot.scatter(xResult,yResult,marker = next(marker),label=key,s=200)
 def extractResultWithParam(cur_res_split,res_names=[],step=0):
     tradeoff_params=cur_res_split.keys()
     tradeoff_nums=[]
@@ -278,6 +373,14 @@ def extractResultWithParam(cur_res_split,res_names=[],step=0):
         res_cur_name=np.array(res_cur_name)[argsort]
         result.append(res_cur_name)
     return result
+def extractResult(cur_res_split,res_names=[],step=0):
+    result=[]
+    for res_name in res_names:
+        res_cur_name=[cur_res_split[res_name][step].tolist()]
+        res_cur_name=np.array(res_cur_name)
+        result.append(res_cur_name)
+    return result
+
 
 def getGrandchildNode(InputDict,GrandchildNode):
     """
@@ -296,4 +399,82 @@ def filteroutNone(result_list):
     for result_list_cur in result_list:
         if result_list_cur[2] is not None:
             result_list_new.append(result_list_cur)
-    return result_list_new
\ No newline at end of file
+    return result_list_new
+def iterate_applyfunction(result,fcn):
+    """
+    This function return iteratively apply function fcn to result if result is a dataframe.
+    """
+    for key in result:
+        if isinstance(result[key], pd.DataFrame):
+            fcn(result[key])
+        else:
+            iterate_applyfunction(result[key],fcn)
+
+def  reorderDict(dictInput,keys):
+    """
+    This function reorder the key of an dict input.
+    """    
+    dictOutput={}
+    for key in keys:
+        if key in dictInput:
+            dictOutput[key]=dictInput[key]
+    for key in dictInput:
+        if key not in dictOutput:
+            dictOutput[key]=dictInput[key]
+    return  dictOutput
+
+def reorder(desiredList,curList):
+    """
+    This function index mapping from the curList according to desiredList.
+    """
+    reoderIndex=[]
+    desiredListCur=[]
+    for legend in desiredList:
+        if legend in curList:
+            desiredListCur.append(legend)
+    desiredList=desiredListCur
+    for legend in curList:
+        if legend not in desiredList:
+            desiredList.append(legend)
+    for legend in desiredList:
+        if legend in curList:
+            reoderIndex.append(curList.index(legend))
+    return reoderIndex
+def reorderLegend(desiredLegend,ax,returnHandles=False):
+    """
+    This function index reorder the legend to desiredLegend.
+    """
+    handles, labels = plt.gca().get_legend_handles_labels()
+    order=reorder(desiredLegend,labels)
+    handles=[handles[idx] for idx in order]
+    labels=[labels[idx] for idx in order]
+    legend=ax.legend(handles,labels)
+    if returnHandles:
+        return legend,handles,labels
+    return legend
+import matplotlib
+def figureConfig():
+    font = {'family' : 'normal',
+            'size'   : 12}
+    matplotlib.rc('font', **font)
+def setScaleFunction(a,b=1,low=True):
+    """
+    This function returns the scale conversion function.
+    """
+    if low:
+        forwad=lambda x: np.log(((x-a)*b))
+        inverse=lambda x: a+(np.exp(x))/b        
+    else:
+        forwad=lambda x: -np.log(((a-x)*b))
+        inverse=lambda x: a-(np.exp(-x))/b
+    return [forwad,inverse]
+def export_legend(legend, filename="legend.png", expand=[-5,-5,5,5]):
+    """
+    This function plot and save the legend.
+    """
+    fig  = legend.figure
+    fig.canvas.draw()
+    bbox  = legend.get_window_extent()
+    bbox = bbox.from_extents(*(bbox.extents + np.array(expand)))
+    bbox = bbox.transformed(fig.dpi_scale_trans.inverted())
+    fig.savefig(filename, dpi=600,pad_inches=0.05, bbox_inches=bbox)
\ No newline at end of file
